@@ -13,16 +13,16 @@ import tempfile
 import zipfile
 
 from bypy.constants import PREFIX, PYTHON, SW, python_major_minor_version
-from bypy.macos_sign import (
-    codesign, create_entitlements_file, make_certificate_useable, notarize_app,
-    verify_signature
-)
-from bypy.utils import current_dir, py_compile, run_shell, timeit, walk
+from bypy.freeze import extract_extension_modules, freeze_python, path_to_freeze_dir
+from bypy.macos_sign import codesign, create_entitlements_file, make_certificate_useable, notarize_app, verify_signature
+from bypy.utils import current_dir, mkdtemp, py_compile, run_shell, timeit, walk
 
 iv = globals()['init_env']
 kitty_constants = iv['kitty_constants']
+self_dir = os.path.dirname(os.path.abspath(__file__))
 join = os.path.join
 basename = os.path.basename
+dirname = os.path.dirname
 abspath = os.path.abspath
 APPNAME = kitty_constants['appname']
 VERSION = kitty_constants['version']
@@ -83,7 +83,7 @@ def strip_files(files, argv_max=(256 * 1024)):
 def files_in(folder):
     for record in os.walk(folder):
         for f in record[-1]:
-            yield os.path.join(record[0], f)
+            yield join(record[0], f)
 
 
 def expand_dirs(items, exclude=lambda x: x.endswith('.so')):
@@ -96,7 +96,7 @@ def expand_dirs(items, exclude=lambda x: x.endswith('.so')):
 
 
 def do_sign(app_dir):
-    with current_dir(os.path.join(app_dir, 'Contents')):
+    with current_dir(join(app_dir, 'Contents')):
         # Sign all .so files
         so_files = {x for x in files_in('.') if x.endswith('.so')}
         codesign(so_files)
@@ -106,6 +106,9 @@ def do_sign(app_dir):
             codesign(fw)
             items = set(os.listdir('.')) - fw
             codesign(expand_dirs(items))
+        # Sign kitten
+        with current_dir('MacOS'):
+            codesign('kitten')
 
     # Now sign the main app
     codesign(app_dir)
@@ -134,8 +137,9 @@ class Freeze(object):
 
     FID = '@executable_path/../Frameworks'
 
-    def __init__(self, build_dir, dont_strip=False, sign_installers=False, notarize=False):
+    def __init__(self, build_dir, dont_strip=False, sign_installers=False, notarize=False, skip_tests=False):
         self.build_dir = build_dir
+        self.skip_tests = skip_tests
         self.sign_installers = sign_installers
         self.notarize = notarize
         self.dont_strip = dont_strip
@@ -145,8 +149,9 @@ class Freeze(object):
         self.to_strip = []
         self.warnings = []
         self.py_ver = py_ver
-        self.python_stdlib = join(self.resources_dir, 'Python', 'lib', 'python' + self.py_ver)
+        self.python_stdlib = join(self.resources_dir, 'Python', 'lib', f'python{self.py_ver}')
         self.site_packages = self.python_stdlib  # hack to avoid needing to add site-packages to path
+        self.obj_dir = mkdtemp('launchers-')
 
         self.run()
 
@@ -160,21 +165,46 @@ class Freeze(object):
         self.add_site_packages()
         self.add_stdlib()
         self.add_misc_libraries()
-        self.compile_py_modules()
-        self.fix_dependencies_in_kitty()
+        self.freeze_python()
+        self.add_ca_certs()
+        self.build_frozen_tools()
         if not self.dont_strip:
             self.strip_files()
+        if not self.skip_tests:
+            self.run_tests()
         # self.run_shell()
 
-        ret = self.makedmg(self.build_dir, APPNAME + '-' + VERSION)
+        ret = self.makedmg(self.build_dir, f'{APPNAME}-{VERSION}')
 
         return ret
 
+    @flush
+    def add_ca_certs(self):
+        print('\nDownloading CA certs...')
+        from urllib.request import urlopen
+        cdata = None
+        for i in range(5):
+            try:
+                cdata = urlopen(kitty_constants['cacerts_url']).read()
+                break
+            except Exception as e:
+                print(f'Downloading CA certs failed with error: {e}, retrying...')
+
+        if cdata is None:
+            raise SystemExit('Downloading C certs failed, giving up')
+        dest = join(self.contents_dir, 'Resources', 'cacert.pem')
+        with open(dest, 'wb') as f:
+            f.write(cdata)
+
     @flush
     def strip_files(self):
         print('\nStripping files...')
         strip_files(self.to_strip)
 
+    @flush
+    def run_tests(self):
+        iv['run_tests'](join(self.contents_dir, 'MacOS', 'kitty'))
+
     @flush
     def set_id(self, path_to_lib, new_id):
         old_mode = flipwritable(path_to_lib)
@@ -198,10 +228,10 @@ class Freeze(object):
     @flush
     def get_local_dependencies(self, path_to_lib):
         for x, is_id in self.get_dependencies(path_to_lib):
-            for y in (PREFIX + '/lib/', PREFIX + '/python/Python.framework/'):
+            for y in (f'{PREFIX}/lib/', f'{PREFIX}/python/Python.framework/', '@rpath/'):
                 if x.startswith(y):
-                    if y == PREFIX + '/python/Python.framework/':
-                        y = PREFIX + '/python/'
+                    if y == f'{PREFIX}/python/Python.framework/':
+                        y = f'{PREFIX}/python/'
                     yield x, x[len(y):], is_id
                     break
 
@@ -215,7 +245,7 @@ class Freeze(object):
         self.to_strip.append(path_to_lib)
         old_mode = flipwritable(path_to_lib)
         for dep, bname, is_id in self.get_local_dependencies(path_to_lib):
-            ndep = self.FID + '/' + bname
+            ndep = f'{self.FID}/{bname}'
             self.change_dep(dep, ndep, is_id, path_to_lib)
         ldeps = list(self.get_local_dependencies(path_to_lib))
         if ldeps:
@@ -228,7 +258,7 @@ class Freeze(object):
     @flush
     def add_python_framework(self):
         print('\nAdding Python framework')
-        src = join(PREFIX + '/python', 'Python.framework')
+        src = join(f'{PREFIX}/python', 'Python.framework')
         x = join(self.frameworks_dir, 'Python.framework')
         curr = os.path.realpath(join(src, 'Versions', 'Current'))
         currd = join(x, 'Versions', basename(curr))
@@ -238,13 +268,12 @@ class Freeze(object):
         shutil.copy2(join(curr, 'Python'), currd)
         self.set_id(
             join(currd, 'Python'),
-            self.FID + '/Python.framework/Versions/%s/Python' % basename(curr))
-        self.fix_dependencies_in_lib(join(self.contents_dir, 'MacOS', 'kitty'))
+            f'{self.FID}/Python.framework/Versions/{basename(curr)}/Python')
         # The following is needed for codesign
         with current_dir(x):
             os.symlink(basename(curr), 'Versions/Current')
             for y in ('Python', 'Resources'):
-                os.symlink('Versions/Current/%s' % y, y)
+                os.symlink(f'Versions/Current/{y}', y)
 
     @flush
     def install_dylib(self, path, set_id=True):
@@ -252,7 +281,7 @@ class Freeze(object):
         if set_id:
             self.set_id(
                 join(self.frameworks_dir, basename(path)),
-                self.FID + '/' + basename(path))
+                f'{self.FID}/{basename(path)}')
         self.fix_dependencies_in_lib(join(self.frameworks_dir, basename(path)))
 
     @flush
@@ -265,19 +294,15 @@ class Freeze(object):
                 'lcms2.2',
                 'crypto.1.1',
                 'ssl.1.1',
+                'xxhash.0',
         ):
             print('\nAdding', x)
-            x = 'lib%s.dylib' % x
+            x = f'lib{x}.dylib'
             src = join(PREFIX, 'lib', x)
             shutil.copy2(src, self.frameworks_dir)
             dest = join(self.frameworks_dir, x)
-            self.set_id(dest, self.FID + '/' + x)
+            self.set_id(dest, f'{self.FID}/{x}')
             self.fix_dependencies_in_lib(dest)
-        base = join(self.frameworks_dir, 'kitty')
-        for lib in walk(base):
-            if lib.endswith('.so'):
-                self.set_id(lib, self.FID + '/' + os.path.relpath(lib, self.frameworks_dir))
-                self.fix_dependencies_in_lib(lib)
 
     @flush
     def add_package_dir(self, x, dest=None):
@@ -295,25 +320,14 @@ class Freeze(object):
             dest = self.site_packages
         dest = join(dest, basename(x))
         shutil.copytree(x, dest, symlinks=True, ignore=ignore)
-        self.postprocess_package(x, dest)
         for f in walk(dest):
             if f.endswith('.so'):
                 self.fix_dependencies_in_lib(f)
 
-    @flush
-    def fix_dependencies_in_kitty(self):
-        for f in walk(join(self.resources_dir, 'kitty')):
-            if f.endswith('.so'):
-                self.fix_dependencies_in_lib(f)
-
-    @flush
-    def postprocess_package(self, src_path, dest_path):
-        pass
-
     @flush
     def add_stdlib(self):
         print('\nAdding python stdlib')
-        src = PREFIX + '/python/Python.framework/Versions/Current/lib/python' + self.py_ver
+        src = f'{PREFIX}/python/Python.framework/Versions/Current/lib/python{self.py_ver}'
         dest = self.python_stdlib
         if not os.path.exists(dest):
             os.makedirs(dest)
@@ -331,6 +345,40 @@ class Freeze(object):
                 if dest2.endswith('.so'):
                     self.fix_dependencies_in_lib(dest2)
 
+    @flush
+    def freeze_python(self):
+        print('\nFreezing python')
+        kitty_dir = join(self.resources_dir, 'kitty')
+        bases = ('kitty', 'kittens', 'kitty_tests')
+        for x in bases:
+            dest = join(self.python_stdlib, x)
+            os.rename(join(kitty_dir, x), dest)
+            if x == 'kitty':
+                shutil.rmtree(join(dest, 'launcher'))
+        os.rename(join(kitty_dir, '__main__.py'), join(self.python_stdlib, 'kitty_main.py'))
+        shutil.rmtree(join(kitty_dir, '__pycache__'))
+        pdir = join(dirname(self.python_stdlib), 'kitty-extensions')
+        os.mkdir(pdir)
+        print('Extracting extension modules from', self.python_stdlib, 'to', pdir)
+        ext_map = extract_extension_modules(self.python_stdlib, pdir)
+        shutil.copy(join(os.path.dirname(self_dir), 'site.py'), join(self.python_stdlib, 'site.py'))
+        for x in bases:
+            iv['sanitize_source_folder'](join(self.python_stdlib, x))
+        self.compile_py_modules()
+        freeze_python(self.python_stdlib, pdir, self.obj_dir, ext_map, develop_mode_env_var='KITTY_DEVELOP_FROM', remove_pyc_files=True)
+        shutil.rmtree(self.python_stdlib)
+        iv['build_frozen_launcher']([path_to_freeze_dir(), self.obj_dir])
+        os.rename(join(dirname(self.contents_dir), 'bin', 'kitty'), join(self.contents_dir, 'MacOS', 'kitty'))
+        shutil.rmtree(join(dirname(self.contents_dir), 'bin'))
+        self.fix_dependencies_in_lib(join(self.contents_dir, 'MacOS', 'kitty'))
+        for f in walk(pdir):
+            if f.endswith('.so') or f.endswith('.dylib'):
+                self.fix_dependencies_in_lib(f)
+
+    @flush
+    def build_frozen_tools(self):
+        iv['build_frozen_tools'](join(self.contents_dir, 'MacOS', 'kitty'))
+
     @flush
     def add_site_packages(self):
         print('\nAdding site-packages')
@@ -389,34 +437,31 @@ class Freeze(object):
 
     @flush
     def compile_py_modules(self):
-        print('\nCompiling Python modules')
         self.remove_bytecode(join(self.resources_dir, 'Python'))
         py_compile(join(self.resources_dir, 'Python'))
-        self.remove_bytecode(join(self.resources_dir, 'kitty'))
-        py_compile(join(self.resources_dir, 'kitty'))
 
     @flush
-    def makedmg(self, d, volname, internet_enable=True, format='ULFO'):
+    def makedmg(self, d, volname, format='ULFO'):
         ''' Copy a directory d into a dmg named volname '''
         print('\nMaking dmg...')
         sys.stdout.flush()
-        destdir = os.path.join(SW, 'dist')
+        destdir = join(SW, 'dist')
         try:
             shutil.rmtree(destdir)
         except FileNotFoundError:
             pass
         os.mkdir(destdir)
-        dmg = os.path.join(destdir, volname + '.dmg')
+        dmg = join(destdir, f'{volname}.dmg')
         if os.path.exists(dmg):
             os.unlink(dmg)
         tdir = tempfile.mkdtemp()
-        appdir = os.path.join(tdir, os.path.basename(d))
+        appdir = join(tdir, os.path.basename(d))
         shutil.copytree(d, appdir, symlinks=True)
         if self.sign_installers:
             with timeit() as times:
                 sign_app(appdir, self.notarize)
-            print('Signing completed in %d minutes %d seconds' % tuple(times))
-        os.symlink('/Applications', os.path.join(tdir, 'Applications'))
+            print('Signing completed in {} minutes {} seconds'.format(*times))
+        os.symlink('/Applications', join(tdir, 'Applications'))
         size_in_mb = int(
             subprocess.check_output(['du', '-s', '-k', tdir]).decode('utf-8')
             .split()[0]) / 1024.
@@ -432,27 +477,22 @@ class Freeze(object):
         print('\nCreating dmg...')
         with timeit() as times:
             subprocess.check_call(cmd + [dmg])
-            if internet_enable:
-                subprocess.check_call(
-                    ['/usr/bin/hdiutil', 'internet-enable', '-yes', dmg])
-        print('dmg created in %d minutes and %d seconds' % tuple(times))
+        print('dmg created in {} minutes and {} seconds'.format(*times))
         shutil.rmtree(tdir)
         size = os.stat(dmg).st_size / (1024 * 1024.)
-        print('\nInstaller size: %.2fMB\n' % size)
+        print(f'\nInstaller size: {size:.2f}MB\n')
         return dmg
 
 
 def main():
     args = globals()['args']
     ext_dir = globals()['ext_dir']
-    if not args.skip_tests:
-        run_tests = iv['run_tests']
-        run_tests(None, os.path.join(ext_dir, 'src'))
     Freeze(
-        os.path.join(ext_dir, kitty_constants['appname'] + '.app'),
+        join(ext_dir, f'{kitty_constants["appname"]}.app'),
         dont_strip=args.dont_strip,
         sign_installers=args.sign_installers,
-        notarize=args.notarize
+        notarize=args.notarize,
+        skip_tests=args.skip_tests
     )
 
 
