@@ -32,6 +32,7 @@ from typing import (
 import kitty.constants as kc
 from kittens.tui.operations import Mode
 from kittens.tui.spinners import spinners
+from kitty.actions import get_all_actions
 from kitty.cli import (
     CompletionSpec,
     GoOption,
@@ -237,13 +238,10 @@ def completion_for_launch_wrappers(*names: str) -> None:
 
 
 def generate_completions_for_kitty() -> None:
-    from kitty.config import option_names_for_completion
     print('package completion\n')
     print('import "kitty/tools/cli"')
     print('import "kitty/tools/cmd/tool"')
     print('import "kitty/tools/cmd/at"')
-    conf_names = ', '.join((f'"{serialize_as_go_string(x)}"' for x in option_names_for_completion()))
-    print('var kitty_option_names_for_completion = []string{' + conf_names + '}')
 
     print('func kitty(root *cli.Command) {')
 
@@ -428,7 +426,7 @@ def go_code_for_remote_command(name: str, cmd: RemoteCommand, template: str) ->
 # kittens {{{
 
 @lru_cache
-def wrapped_kittens() -> Sequence[str]:
+def wrapped_kittens() -> Tuple[str, ...]:
     with open('shell-integration/ssh/kitty') as f:
         for line in f:
             if line.startswith('    wrapped_kittens="'):
@@ -465,7 +463,7 @@ def generate_extra_cli_parser(name: str, spec: str) -> None:
 
 def kitten_clis() -> None:
     from kittens.runner import get_kitten_conf_docs, get_kitten_extra_cli_parsers
-    for kitten in wrapped_kittens():
+    for kitten in wrapped_kittens() + ('pager',):
         defn = get_kitten_conf_docs(kitten)
         if defn is not None:
             generate_conf_parser(kitten, defn)
@@ -572,6 +570,7 @@ def load_ref_map() -> Dict[str, Dict[str, str]]:
 
 def generate_constants() -> str:
     from kittens.hints.main import DEFAULT_REGEX
+    from kitty.config import option_names_for_completion
     from kitty.fast_data_types import FILE_TRANSFER_CODE
     from kitty.options.utils import allowed_shell_integration_values
     del sys.modules['kittens.hints.main']
@@ -582,6 +581,7 @@ def generate_constants() -> str:
         placeholder_char = int(m.group(1), 16)
     dp = ", ".join(map(lambda x: f'"{serialize_as_go_string(x)}"', kc.default_pager_for_help))
     url_prefixes = ','.join(f'"{x}"' for x in Options.url_prefixes)
+    option_names = '`' + '\n'.join(option_names_for_completion()) + '`'
     return f'''\
 package kitty
 
@@ -609,14 +609,15 @@ var RefMap = map[string]string{serialize_go_dict(ref_map['ref'])}
 var DocTitleMap = map[string]string{serialize_go_dict(ref_map['doc'])}
 var AllowedShellIntegrationValues = []string{{ {str(sorted(allowed_shell_integration_values))[1:-1].replace("'", '"')} }}
 var KittyConfigDefaults = struct {{
-Term, Shell_integration, Select_by_word_characters, Shell string
+Term, Shell_integration, Select_by_word_characters, Url_excluded_characters, Shell string
 Wheel_scroll_multiplier int
 Url_prefixes []string
 }}{{
 Term: "{Options.term}", Shell_integration: "{' '.join(Options.shell_integration)}", Url_prefixes: []string{{ {url_prefixes} }},
 Select_by_word_characters: `{Options.select_by_word_characters}`, Wheel_scroll_multiplier: {Options.wheel_scroll_multiplier},
-Shell: "{Options.shell}",
+Shell: "{Options.shell}", Url_excluded_characters: "{Options.url_excluded_characters}",
 }}
+const OptionNames = {option_names}
 '''  # }}}
 
 
@@ -688,6 +689,15 @@ func add_rc_global_opts(cmd *cli.Command) {{
 def update_completion() -> None:
     with replace_if_needed('tools/cmd/completion/kitty_generated.go'):
         generate_completions_for_kitty()
+
+    with replace_if_needed('tools/cmd/at/kitty_actions_generated.go'):
+        print("package at")
+        print("const KittyActionNames = `", end='')
+        for grp, actions in get_all_actions().items():
+            for ac in actions:
+                print(ac.name)
+        print('`')
+
     with replace_if_needed('tools/cmd/edit_in_kitty/launch_generated.go'):
         print('package edit_in_kitty')
         print('import "kitty/tools/cli"')
