@@ -308,6 +308,10 @@ static NSDictionary<NSString*,NSNumber*> *global_shortcuts = nil;
     return NSTerminateCancel;
 }
 
+- (BOOL)applicationSupportsSecureRestorableState:(NSApplication *)app {
+    return YES;
+}
+
 static GLFWapplicationshouldhandlereopenfun handle_reopen_callback = NULL;
 
 - (BOOL)applicationShouldHandleReopen:(NSApplication *)sender hasVisibleWindows:(BOOL)flag
@@ -770,6 +774,11 @@ is_useful_apple_global_shortcut(int sc) {
     }
 }
 
+static bool
+is_apple_jis_layout_function_key(NSEvent *event) {
+    return [event keyCode] == 0x66 /* kVK_JIS_Eisu */ || [event keyCode] == 0x68 /* kVK_JIS_Kana */;
+}
+
 GLFWAPI GLFWapplicationshouldhandlereopenfun glfwSetApplicationShouldHandleReopen(GLFWapplicationshouldhandlereopenfun callback) {
     GLFWapplicationshouldhandlereopenfun previous = handle_reopen_callback;
     handle_reopen_callback = callback;
@@ -821,7 +830,7 @@ int _glfwPlatformInit(void)
         if (!_glfw.ignoreOSKeyboardProcessing) {
             // first check if there is a global menu bar shortcut
             if ([[NSApp mainMenu] performKeyEquivalent:event]) {
-                debug_key("keyDown triggerred global menu bar action ignoring\n");
+                debug_key("keyDown triggered global menu bar action ignoring\n");
                 last_keydown_shortcut_event.virtual_key_code = [event keyCode];
                 last_keydown_shortcut_event.input_source_switch_modifiers = 0;
                 last_keydown_shortcut_event.timestamp = [event timestamp];
@@ -830,13 +839,21 @@ int _glfwPlatformInit(void)
             // now check if there is a useful apple shortcut
             int global_shortcut = is_active_apple_global_shortcut(event);
             if (is_useful_apple_global_shortcut(global_shortcut)) {
-                debug_key("keyDown triggerred global macOS shortcut ignoring\n");
+                debug_key("keyDown triggered global macOS shortcut ignoring\n");
                 last_keydown_shortcut_event.virtual_key_code = [event keyCode];
                 // record the modifier keys if switching to the next input source
                 last_keydown_shortcut_event.input_source_switch_modifiers = (global_shortcut == kSHKSelectNextSourceInInputMenu) ? USEFUL_MODS([event modifierFlags]) : 0;
                 last_keydown_shortcut_event.timestamp = [event timestamp];
                 return event;
             }
+            // check for JIS keyboard layout function keys
+            if (is_apple_jis_layout_function_key(event)) {
+                debug_key("keyDown triggered JIS layout function key ignoring\n");
+                last_keydown_shortcut_event.virtual_key_code = [event keyCode];
+                last_keydown_shortcut_event.input_source_switch_modifiers = 0;
+                last_keydown_shortcut_event.timestamp = [event timestamp];
+                return event;
+            }
         }
         last_keydown_shortcut_event.virtual_key_code = 0xffff;
         NSWindow *kw = [NSApp keyWindow];
@@ -852,7 +869,7 @@ int _glfwPlatformInit(void)
         if (last_keydown_shortcut_event.virtual_key_code != 0xffff && last_keydown_shortcut_event.virtual_key_code == [event keyCode]) {
             // ignore as the corresponding key down event triggered a menu bar or macOS shortcut
             last_keydown_shortcut_event.virtual_key_code = 0xffff;
-            debug_key("keyUp ignored as corresponds to previous keyDown that trigerred a shortcut\n");
+            debug_key("keyUp ignored as corresponds to previous keyDown that triggered a shortcut\n");
             return nil;
         }
         NSWindow *kw = [NSApp keyWindow];
@@ -979,8 +996,6 @@ void _glfwPlatformTerminate(void)
         _glfw.ns.appleSettings = nil;
     }
 
-    free(_glfw.ns.clipboardString);
-
     _glfwTerminateNSGL();
     if (global_shortcuts != nil) { [global_shortcuts release]; global_shortcuts = nil; }
 
@@ -1003,19 +1018,21 @@ static pthread_t main_thread;
 static NSLock *tick_lock = NULL;
 
 
-void _glfwDispatchTickCallback() {
+void _glfwDispatchTickCallback(void) {
     if (tick_lock && tick_callback) {
-        [tick_lock lock];
-        while(tick_callback_requested) {
-            tick_callback_requested = false;
-            tick_callback(tick_callback_data);
+        while(true) {
+            bool do_call = false;
+            [tick_lock lock];
+            if (tick_callback_requested) { do_call = true; tick_callback_requested = false; }
+            [tick_lock unlock];
+            if (do_call) tick_callback(tick_callback_data);
+            else break;
         }
-        [tick_lock unlock];
     }
 }
 
 static void
-request_tick_callback() {
+request_tick_callback(void) {
     if (!tick_callback_requested) {
         tick_callback_requested = true;
         [NSApp performSelectorOnMainThread:@selector(tick_callback) withObject:nil waitUntilDone:NO];
