@@ -30,7 +30,6 @@
 
 #include "internal.h"
 #include "backend_utils.h"
-#include "memfd.h"
 #include "linux_notify.h"
 #include "wl_client_side_decorations.h"
 #include "../kitty/monotonic.h"
@@ -152,10 +151,58 @@ static struct wl_buffer* createShmBuffer(const GLFWimage* image, bool is_opaque,
     return buffer;
 }
 
+static int
+glfw_cursor_shape_to_wayland_cursor_shape(GLFWCursorShape g) {
+#define C(g, w) case g: return w;
+    switch(g) {
+        /* start glfw to wayland mapping (auto generated by gen-key-constants.py do not edit) */
+        C(GLFW_DEFAULT_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_DEFAULT);
+        C(GLFW_TEXT_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_TEXT);
+        C(GLFW_POINTER_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_POINTER);
+        C(GLFW_HELP_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_HELP);
+        C(GLFW_WAIT_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_WAIT);
+        C(GLFW_PROGRESS_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_PROGRESS);
+        C(GLFW_CROSSHAIR_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_CROSSHAIR);
+        C(GLFW_CELL_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_CELL);
+        C(GLFW_VERTICAL_TEXT_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_VERTICAL_TEXT);
+        C(GLFW_MOVE_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_MOVE);
+        C(GLFW_E_RESIZE_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_E_RESIZE);
+        C(GLFW_NE_RESIZE_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_NE_RESIZE);
+        C(GLFW_NW_RESIZE_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_NW_RESIZE);
+        C(GLFW_N_RESIZE_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_N_RESIZE);
+        C(GLFW_SE_RESIZE_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_SE_RESIZE);
+        C(GLFW_SW_RESIZE_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_SW_RESIZE);
+        C(GLFW_S_RESIZE_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_S_RESIZE);
+        C(GLFW_W_RESIZE_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_W_RESIZE);
+        C(GLFW_EW_RESIZE_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_EW_RESIZE);
+        C(GLFW_NS_RESIZE_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_NS_RESIZE);
+        C(GLFW_NESW_RESIZE_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_NESW_RESIZE);
+        C(GLFW_NWSE_RESIZE_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_NWSE_RESIZE);
+        C(GLFW_ZOOM_IN_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_ZOOM_IN);
+        C(GLFW_ZOOM_OUT_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_ZOOM_OUT);
+        C(GLFW_ALIAS_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_ALIAS);
+        C(GLFW_COPY_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_COPY);
+        C(GLFW_NOT_ALLOWED_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_NOT_ALLOWED);
+        C(GLFW_NO_DROP_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_NO_DROP);
+        C(GLFW_GRAB_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_GRAB);
+        C(GLFW_GRABBING_CURSOR, WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_GRABBING);
+/* end glfw to wayland mapping */
+        default: return -1;
+    }
+#undef C
+}
+
 static void
 setCursorImage(_GLFWwindow* window, bool on_theme_change) {
     _GLFWcursorWayland defaultCursor = {.shape = GLFW_DEFAULT_CURSOR};
     _GLFWcursorWayland* cursorWayland = window->cursor ? &window->cursor->wl : &defaultCursor;
+    if (_glfw.wl.wp_cursor_shape_device_v1) {
+        int which = glfw_cursor_shape_to_wayland_cursor_shape(cursorWayland->shape);
+        if (which > -1) {
+            wp_cursor_shape_device_v1_set_shape(_glfw.wl.wp_cursor_shape_device_v1, _glfw.wl.pointer_enter_serial, (uint32_t)which);
+            return;
+        }
+    }
     struct wl_cursor_image* image = NULL;
     struct wl_buffer* buffer = NULL;
     struct wl_surface* surface = _glfw.wl.cursorSurface;
@@ -717,7 +764,7 @@ static void incrementCursorImage(_GLFWwindow* window)
 {
     if (window && window->wl.decorations.focus == CENTRAL_WINDOW && window->cursorMode != GLFW_CURSOR_HIDDEN) {
         _GLFWcursor* cursor = window->wl.currentCursor;
-        if (cursor && cursor->wl.cursor)
+        if (cursor && cursor->wl.cursor && cursor->wl.cursor->image_count)
         {
             cursor->wl.currentImage += 1;
             cursor->wl.currentImage %= cursor->wl.cursor->image_count;
@@ -2034,9 +2081,19 @@ _glfwPlatformSetClipboard(GLFWClipboardType t) {
         f(data_source, cd->mime_types[i]);
     }
     if (t == GLFW_CLIPBOARD) {
-        // According to the Wayland spec only the application that has keyboard focus can set the clipboard.
+        // According to some interpretations of the Wayland spec only the application that has keyboard focus can set the clipboard.
         // Hurray for the Wayland nanny state!
-        wl_data_device_set_selection(_glfw.wl.dataDevice, _glfw.wl.dataSourceForClipboard, _glfw.wl.keyboard_enter_serial);
+        //
+        // However in wl-roots based compositors, using the serial from the keyboard enter event doesn't work. No clue what
+        // the correct serial to use here is. Given this Wayland there probably isn't one. What a joke.
+        // Bug report: https://github.com/kovidgoyal/kitty/issues/6890
+        // Ironically one of the contributors to wl_roots claims the keyboard enter serial is the correct one to use:
+        // https://emersion.fr/blog/2020/wayland-clipboard-drag-and-drop/
+        // The Wayland spec itself says "serial number of the event that triggered this request"
+        // https://wayland.freedesktop.org/docs/html/apa.html#protocol-spec-wl_data_device
+        // So who the fuck knows. Just use the latest received serial and ask anybody that uses Wayland
+        // to get their head examined.
+        wl_data_device_set_selection(_glfw.wl.dataDevice, _glfw.wl.dataSourceForClipboard, _glfw.wl.serial);
     } else {
         // According to the Wayland spec we can only set the primary selection in response to a pointer button event
         // Hurray for the Wayland nanny state!
@@ -2274,3 +2331,8 @@ GLFWAPI bool glfwWaylandSetTitlebarColor(GLFWwindow *handle, uint32_t color, boo
     }
     return false;
 }
+
+GLFWAPI void glfwWaylandRedrawCSDWindowTitle(GLFWwindow *handle) {
+    _GLFWwindow* window = (_GLFWwindow*) handle;
+    change_csd_title(window);
+}
