@@ -11,11 +11,9 @@
 #include "threading.h"
 #include "screen.h"
 #include "fonts.h"
-#include "charsets.h"
 #include "monotonic.h"
 #include <termios.h>
 #include <unistd.h>
-#include <float.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include <sys/wait.h>
