@@ -212,6 +212,16 @@ find_app_name(void) {
 @end
 // }}}
 
+@interface UserMenuItem : NSMenuItem
+@property (nonatomic) size_t action_index;
+@end
+
+@implementation UserMenuItem {
+}
+@end
+
+
+
 @interface GlobalMenuTarget : NSObject
 + (GlobalMenuTarget *) shared_instance;
 @end
@@ -220,6 +230,13 @@ find_app_name(void) {
 
 @implementation GlobalMenuTarget
 
+- (void)user_menu_action:(id)sender {
+    UserMenuItem *m = sender;
+    if (m.action_index < OPT(global_menu).count && OPT(global_menu.entries)) {
+        set_cocoa_pending_action(USER_MENU_ACTION, OPT(global_menu).entries[m.action_index].definition);
+    }
+}
+
 PENDING(edit_config_file, PREFERENCES_WINDOW)
 PENDING(new_os_window, NEW_OS_WINDOW)
 PENDING(detach_tab, DETACH_TAB)
@@ -235,17 +252,31 @@ PENDING(clear_terminal_and_scrollback, CLEAR_TERMINAL_AND_SCROLLBACK)
 PENDING(reload_config, RELOAD_CONFIG)
 PENDING(toggle_macos_secure_keyboard_entry, TOGGLE_MACOS_SECURE_KEYBOARD_ENTRY)
 PENDING(toggle_fullscreen, TOGGLE_FULLSCREEN)
-
-- (void)open_kitty_website_url:(id)sender {
-    (void)sender;
-    [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"https://sw.kovidgoyal.net/kitty/"]];
-}
+PENDING(open_kitty_website, OPEN_KITTY_WEBSITE)
+PENDING(hide_macos_app, HIDE)
+PENDING(hide_macos_other_apps, HIDE_OTHERS)
+PENDING(minimize_macos_window, MINIMIZE)
+PENDING(quit, QUIT)
 
 - (BOOL)validateMenuItem:(NSMenuItem *)item {
     if (item.action == @selector(toggle_macos_secure_keyboard_entry:)) {
         item.state = [SecureKeyboardEntryController sharedInstance].isDesired ? NSControlStateValueOn : NSControlStateValueOff;
     } else if (item.action == @selector(toggle_fullscreen:)) {
         item.title = ([NSApp currentSystemPresentationOptions] & NSApplicationPresentationFullScreen) ? @"Exit Full Screen" : @"Enter Full Screen";
+        if (![NSApp keyWindow]) return NO;
+    } else if (item.action == @selector(minimize_macos_window:)) {
+        NSWindow *window = [NSApp keyWindow];
+        if (!window || window.miniaturized || [NSApp currentSystemPresentationOptions] & NSApplicationPresentationFullScreen) return NO;
+    } else if (item.action == @selector(close_os_window:) ||
+        item.action == @selector(close_tab:) ||
+        item.action == @selector(close_window:) ||
+        item.action == @selector(reset_terminal:) ||
+        item.action == @selector(clear_terminal_and_scrollback:) ||
+        item.action == @selector(previous_tab:) ||
+        item.action == @selector(next_tab:) ||
+        item.action == @selector(detach_tab:))
+    {
+        if (![NSApp keyWindow]) return NO;
     }
     return YES;
 }
@@ -275,7 +306,8 @@ typedef struct {
 typedef struct {
     GlobalShortcut new_os_window, close_os_window, close_tab, edit_config_file, reload_config;
     GlobalShortcut previous_tab, next_tab, new_tab, new_window, close_window, reset_terminal, clear_terminal_and_scrollback;
-    GlobalShortcut toggle_macos_secure_keyboard_entry, toggle_fullscreen;
+    GlobalShortcut toggle_macos_secure_keyboard_entry, toggle_fullscreen, open_kitty_website;
+    GlobalShortcut hide_macos_app, hide_macos_other_apps, minimize_macos_window, quit;
 } GlobalShortcuts;
 static GlobalShortcuts global_shortcuts;
 
@@ -290,7 +322,8 @@ cocoa_set_global_shortcut(PyObject *self UNUSED, PyObject *args) {
     Q(new_os_window); else Q(close_os_window); else Q(close_tab); else Q(edit_config_file);
     else Q(new_tab); else Q(next_tab); else Q(previous_tab);
     else Q(new_window); else Q(close_window); else Q(reset_terminal); else Q(clear_terminal_and_scrollback); else Q(reload_config);
-    else Q(toggle_macos_secure_keyboard_entry); else Q(toggle_fullscreen);
+    else Q(toggle_macos_secure_keyboard_entry); else Q(toggle_fullscreen); else Q(open_kitty_website);
+    else Q(hide_macos_app); else Q(hide_macos_other_apps); else Q(minimize_macos_window); else Q(quit);
 #undef Q
     if (gs == NULL) { PyErr_SetString(PyExc_KeyError, "Unknown shortcut name"); return NULL; }
     int cocoa_mods;
@@ -499,27 +532,30 @@ cocoa_send_notification(PyObject *self UNUSED, PyObject *args) {
 
 - (BOOL)openTab:(NSPasteboard*)pasteboard
         userData:(NSString *) UNUSED userData error:(NSError **) UNUSED error {
-    return [self openFilesFromPasteboard:pasteboard type:NEW_TAB_WITH_WD];
+    return [self openDirsFromPasteboard:pasteboard type:NEW_TAB_WITH_WD];
 }
 
 - (BOOL)openOSWindow:(NSPasteboard*)pasteboard
         userData:(NSString *) UNUSED userData  error:(NSError **) UNUSED error {
-    return [self openFilesFromPasteboard:pasteboard type:NEW_OS_WINDOW_WITH_WD];
+    return [self openDirsFromPasteboard:pasteboard type:NEW_OS_WINDOW_WITH_WD];
 }
 
-- (BOOL)openFilesFromPasteboard:(NSPasteboard *)pasteboard type:(int)type {
+- (BOOL)openDirsFromPasteboard:(NSPasteboard *)pasteboard type:(int)type {
     NSDictionary *options = @{ NSPasteboardURLReadingFileURLsOnlyKey: @YES };
     NSArray *filePathArray = [pasteboard readObjectsForClasses:[NSArray arrayWithObject:[NSURL class]] options:options];
+    NSMutableArray<NSString*> *dirPathArray = [NSMutableArray arrayWithCapacity:[filePathArray count]];
     for (NSURL *url in filePathArray) {
         NSString *path = [url path];
         BOOL isDirectory = NO;
         if ([[NSFileManager defaultManager] fileExistsAtPath:path isDirectory:&isDirectory]) {
-            if (!isDirectory) {
-                path = [path stringByDeletingLastPathComponent];
-            }
-            set_cocoa_pending_action(type, [path UTF8String]);
+            if (!isDirectory) path = [path stringByDeletingLastPathComponent];
+            if (![dirPathArray containsObject:path]) [dirPathArray addObject:path];
         }
     }
+    if ([dirPathArray count] > 0) {
+        // Colons are not valid in paths under macOS.
+        set_cocoa_pending_action(type, [[dirPathArray componentsJoinedByString:@":"] UTF8String]);
+    }
     return YES;
 }
 
@@ -539,6 +575,36 @@ cocoa_send_notification(PyObject *self UNUSED, PyObject *args) {
 @end
 
 // global menu {{{
+
+static void
+add_user_global_menu_entry(struct MenuItem *e, NSMenu *bar, size_t action_index) {
+    NSMenu *parent = bar;
+    UserMenuItem *final_item = nil;
+    GlobalMenuTarget *global_menu_target = [GlobalMenuTarget shared_instance];
+    for (size_t i = 0; i < e->location_count; i++) {
+        NSMenuItem *item = [parent itemWithTitle:@(e->location[i])];
+        if (!item) {
+            final_item = [[UserMenuItem alloc] initWithTitle:@(e->location[i]) action:@selector(user_menu_action:) keyEquivalent:@""];
+            final_item.target = global_menu_target;
+            [parent addItem:final_item];
+            item = final_item;
+            [final_item release];
+        }
+        if (i + 1 < e->location_count) {
+            if (![item hasSubmenu]) {
+                NSMenu* sub_menu = [[NSMenu alloc] initWithTitle:item.title];
+                [item setSubmenu:sub_menu];
+                [sub_menu release];
+            }
+            parent = [item submenu];
+            if (!parent) return;
+        }
+    }
+    if (final_item != nil) {
+        final_item.action_index = action_index;
+    }
+}
+
 void
 cocoa_create_global_menu(void) {
     NSString* app_name = find_app_name();
@@ -575,13 +641,8 @@ cocoa_create_global_menu(void) {
     [servicesMenu release];
     [appMenu addItem:[NSMenuItem separatorItem]];
 
-    [appMenu addItemWithTitle:[NSString stringWithFormat:@"Hide %@", app_name]
-                       action:@selector(hide:)
-                keyEquivalent:@"h"];
-    [[appMenu addItemWithTitle:@"Hide Others"
-                        action:@selector(hideOtherApplications:)
-                 keyEquivalent:@"h"]
-        setKeyEquivalentModifierMask:NSEventModifierFlagOption | NSEventModifierFlagCommand];
+    MENU_ITEM(appMenu, ([NSString stringWithFormat:@"Hide %@", app_name]), hide_macos_app);
+    MENU_ITEM(appMenu, @"Hide Others", hide_macos_other_apps);
     [appMenu addItemWithTitle:@"Show All"
                        action:@selector(unhideAllApplications:)
                 keyEquivalent:@""];
@@ -590,9 +651,7 @@ cocoa_create_global_menu(void) {
     MENU_ITEM(appMenu, @"Secure Keyboard Entry", toggle_macos_secure_keyboard_entry);
     [appMenu addItem:[NSMenuItem separatorItem]];
 
-    [appMenu addItemWithTitle:[NSString stringWithFormat:@"Quit %@", app_name]
-                       action:@selector(terminate:)
-                keyEquivalent:@"q"];
+    MENU_ITEM(appMenu, ([NSString stringWithFormat:@"Quit %@", app_name]), quit);
     [appMenu release];
 
     NSMenuItem* shellMenuItem =
@@ -614,15 +673,13 @@ cocoa_create_global_menu(void) {
     [shellMenu release];
 
     NSMenuItem* windowMenuItem =
-        [bar addItemWithTitle:@""
+        [bar addItemWithTitle:@"Window"
                        action:NULL
                 keyEquivalent:@""];
     NSMenu* windowMenu = [[NSMenu alloc] initWithTitle:@"Window"];
     [windowMenuItem setSubmenu:windowMenu];
 
-    [windowMenu addItemWithTitle:@"Minimize"
-                          action:@selector(performMiniaturize:)
-                   keyEquivalent:@"m"];
+    MENU_ITEM(windowMenu, @"Minimize", minimize_macos_window);
     [windowMenu addItemWithTitle:@"Zoom"
                           action:@selector(performZoom:)
                    keyEquivalent:@""];
@@ -649,15 +706,22 @@ cocoa_create_global_menu(void) {
                 keyEquivalent:@""];
     NSMenu* helpMenu = [[NSMenu alloc] initWithTitle:@"Help"];
     [helpMenuItem setSubmenu:helpMenu];
-    [[helpMenu addItemWithTitle:[NSString stringWithFormat:@"Visit %@ Website", app_name]
-                         action:@selector(open_kitty_website_url:)
-                  keyEquivalent:@"?"]
-                      setTarget:global_menu_target];
+
+    MENU_ITEM(helpMenu, @"Visit kitty Website", open_kitty_website);
     [NSApp setHelpMenu:helpMenu];
     [helpMenu release];
 
+    if (OPT(global_menu.entries)) {
+        for (size_t i = 0; i < OPT(global_menu.count); i++) {
+            struct MenuItem *e = OPT(global_menu.entries) + i;
+            if (e->definition && e->location && e->location_count > 1) {
+                add_user_global_menu_entry(e, bar, i);
+            }
+        }
+    }
     [bar release];
 
+
     class_addMethod(
         object_getClass([NSApp delegate]),
         @selector(applicationDockMenu:),
@@ -666,6 +730,7 @@ cocoa_create_global_menu(void) {
 
 
     [NSApp setServicesProvider:[[[ServiceProvider alloc] init] autorelease]];
+
 #undef MENU_ITEM
 }
 
@@ -675,7 +740,7 @@ cocoa_update_menu_bar_title(PyObject *pytitle) {
     if (OPT(macos_menubar_title_max_length) > 0 && PyUnicode_GetLength(pytitle) > OPT(macos_menubar_title_max_length)) {
         static char fmt[64];
         snprintf(fmt, sizeof(fmt), "%%%ld.%ldU%%s", OPT(macos_menubar_title_max_length), OPT(macos_menubar_title_max_length));
-        DECREF_AFTER_FUNCTION PyObject *st = PyUnicode_FromFormat(fmt, pytitle, "…");
+        RAII_PyObject(st, PyUnicode_FromFormat(fmt, pytitle, "…"));
         if (st) title = @(PyUnicode_AsUTF8(st));
     } else {
         title = @(PyUnicode_AsUTF8(pytitle));
@@ -689,27 +754,26 @@ cocoa_update_menu_bar_title(PyObject *pytitle) {
     NSMenu *m = [[NSMenu alloc] initWithTitle:[NSString stringWithFormat:@" :: %@", title]];
     [title_menu setSubmenu:m];
     [m release];
-} // }}}
-
-bool
-cocoa_make_window_resizable(void *w, bool resizable) {
-    NSWindow *window = (NSWindow*)w;
-
-    @try {
-        if (resizable) {
-            [window setStyleMask:
-                [window styleMask] | NSWindowStyleMaskResizable];
-        } else {
-            [window setStyleMask:
-                [window styleMask] & ~NSWindowStyleMaskResizable];
-        }
-    } @catch (NSException *e) {
-        log_error("Failed to set style mask: %s: %s", [[e name] UTF8String], [[e reason] UTF8String]);
-        return false;
-    }
-    return true;
 }
 
+void
+cocoa_clear_global_shortcuts(void) {
+    memset(&global_shortcuts, 0, sizeof(global_shortcuts));
+}
+
+void
+cocoa_recreate_global_menu(void) {
+    if (title_menu != NULL) {
+        NSMenu *bar = [NSApp mainMenu];
+        [bar removeItem:title_menu];
+    }
+    title_menu = NULL;
+    cocoa_create_global_menu();
+}
+
+
+// }}}
+
 #define NSLeftAlternateKeyMask  (0x000020 | NSEventModifierFlagOption)
 #define NSRightAlternateKeyMask (0x000040 | NSEventModifierFlagOption)
 
@@ -726,6 +790,22 @@ cocoa_toggle_secure_keyboard_entry(void) {
     [[NSUserDefaults standardUserDefaults] setBool:k.isDesired forKey:@"SecureKeyboardEntry"];
 }
 
+void
+cocoa_hide(void) {
+    [[NSApplication sharedApplication] performSelectorOnMainThread:@selector(hide:) withObject:nil waitUntilDone:NO];
+}
+
+void
+cocoa_hide_others(void) {
+    [[NSApplication sharedApplication] performSelectorOnMainThread:@selector(hideOtherApplications:) withObject:nil waitUntilDone:NO];
+}
+
+void
+cocoa_minimize(void *w) {
+    NSWindow *window = (NSWindow*)w;
+    if (window && !window.miniaturized) [window performSelectorOnMainThread:@selector(performMiniaturize:) withObject:nil waitUntilDone:NO];
+}
+
 void
 cocoa_focus_window(void *w) {
     NSWindow *window = (NSWindow*)w;
@@ -758,19 +838,10 @@ cocoa_get_workspace_ids(void *w, size_t *workspace_ids, size_t array_sz) {
 static PyObject*
 cocoa_get_lang(PyObject UNUSED *self) {
     @autoreleasepool {
-
-    NSString* locale = nil;
-    NSString* lang_code = [[NSLocale currentLocale] objectForKey:NSLocaleLanguageCode];
+    NSString* lang_code = [[NSLocale currentLocale] languageCode];
     NSString* country_code = [[NSLocale currentLocale] objectForKey:NSLocaleCountryCode];
-    if (lang_code && country_code) {
-        locale = [NSString stringWithFormat:@"%@_%@", lang_code, country_code];
-    } else {
-        locale = [[NSLocale currentLocale] localeIdentifier];
-    }
-    if (!locale) { Py_RETURN_NONE; }
-    const char* locale_utf8 = [locale UTF8String];
-    return Py_BuildValue("s", locale_utf8);
-
+    NSString* identifier = [[NSLocale currentLocale] localeIdentifier];
+    return Py_BuildValue("sss", lang_code ? [lang_code UTF8String]:"", country_code ? [country_code UTF8String] : "", identifier ? [identifier UTF8String]: "");
     } // autoreleasepool
 }
 
@@ -798,46 +869,6 @@ cocoa_set_activation_policy(bool hide_from_tasks) {
     [NSApp setActivationPolicy:(hide_from_tasks ? NSApplicationActivationPolicyAccessory : NSApplicationActivationPolicyRegular)];
 }
 
-void
-cocoa_set_titlebar_appearance(void *w, unsigned int theme)
-{
-    if (!theme) return;
-    @autoreleasepool {
-        NSWindow *window = (NSWindow*)w;
-        [window setAppearance:[NSAppearance appearanceNamed:((theme == 2) ? NSAppearanceNameVibrantDark : NSAppearanceNameVibrantLight)]];
-    } // autoreleasepool
-}
-
-void
-cocoa_set_titlebar_color(void *w, color_type titlebar_color)
-{
-    @autoreleasepool {
-
-    NSWindow *window = (NSWindow*)w;
-
-    double red = ((titlebar_color >> 16) & 0xFF) / 255.0;
-    double green = ((titlebar_color >> 8) & 0xFF) / 255.0;
-    double blue = (titlebar_color & 0xFF) / 255.0;
-
-    NSColor *background =
-        [NSColor colorWithSRGBRed:red
-                            green:green
-                             blue:blue
-                            alpha:1.0];
-    [window setTitlebarAppearsTransparent:YES];
-    [window setBackgroundColor:background];
-
-    double luma = 0.2126 * red + 0.7152 * green + 0.0722 * blue;
-
-    if (luma < 0.5) {
-        [window setAppearance:[NSAppearance appearanceNamed:NSAppearanceNameVibrantDark]];
-    } else {
-        [window setAppearance:[NSAppearance appearanceNamed:NSAppearanceNameVibrantLight]];
-    }
-
-    } // autoreleasepool
-}
-
 static PyObject*
 cocoa_set_url_handler(PyObject UNUSED *self, PyObject *args) {
     @autoreleasepool {
@@ -865,10 +896,78 @@ cocoa_set_url_handler(PyObject UNUSED *self, PyObject *args) {
     } // autoreleasepool
 }
 
+static PyObject*
+cocoa_set_app_icon(PyObject UNUSED *self, PyObject *args) {
+    @autoreleasepool {
+
+    const char *icon_path = NULL, *app_path = NULL;
+    if (!PyArg_ParseTuple(args, "s|z", &icon_path, &app_path)) return NULL;
+    if (!icon_path || icon_path[0] == '\0') {
+        PyErr_SetString(PyExc_TypeError, "Empty icon file path");
+        return NULL;
+    }
+    NSString *custom_icon_path = [NSString stringWithUTF8String:icon_path];
+    if (![[NSFileManager defaultManager] fileExistsAtPath:custom_icon_path]) {
+        PyErr_Format(PyExc_FileNotFoundError, "Icon file not found: %s", [custom_icon_path UTF8String]);
+        return NULL;
+    }
+
+    NSString *bundle_path = @"";
+    if (!app_path) {
+        bundle_path = [[NSBundle mainBundle] bundlePath];
+        if (!bundle_path || bundle_path.length == 0) bundle_path = @"/Applications/kitty.app";
+        // When compiled from source and run from the launcher folder the bundle path should be `kitty.app` in it
+        if (![bundle_path hasSuffix:@".app"]) {
+            NSString *launcher_app_path = [bundle_path stringByAppendingPathComponent:@"kitty.app"];
+            bundle_path = @"";
+            BOOL is_dir;
+            if ([[NSFileManager defaultManager] fileExistsAtPath:launcher_app_path isDirectory:&is_dir] && is_dir && [[NSWorkspace sharedWorkspace] isFilePackageAtPath:launcher_app_path]) {
+                bundle_path = launcher_app_path;
+            }
+        }
+    } else if (app_path[0] != '\0') {
+        bundle_path = [NSString stringWithUTF8String:app_path];
+    }
+    if (!bundle_path || bundle_path.length == 0 || ![[NSFileManager defaultManager] fileExistsAtPath:bundle_path]) {
+        PyErr_Format(PyExc_FileNotFoundError, "Application bundle not found: %s", [bundle_path UTF8String]);
+        return NULL;
+    }
+
+    NSImage *icon_image = [[NSImage alloc] initWithContentsOfFile:custom_icon_path];
+    BOOL result = [[NSWorkspace sharedWorkspace] setIcon:icon_image forFile:bundle_path options:NSExcludeQuickDrawElementsIconCreationOption];
+    [icon_image release];
+    if (result) Py_RETURN_NONE;
+    PyErr_Format(PyExc_OSError, "Failed to set custom icon %s for %s", [custom_icon_path UTF8String], [bundle_path UTF8String]);
+    return NULL;
+
+    } // autoreleasepool
+}
+
+static PyObject*
+cocoa_set_dock_icon(PyObject UNUSED *self, PyObject *args) {
+    @autoreleasepool {
+
+    const char *icon_path = NULL;
+    if (!PyArg_ParseTuple(args, "s", &icon_path)) return NULL;
+    if (!icon_path || icon_path[0] == '\0') {
+        PyErr_SetString(PyExc_TypeError, "Empty icon file path");
+        return NULL;
+    }
+    NSString *custom_icon_path = [NSString stringWithUTF8String:icon_path];
+    if ([[NSFileManager defaultManager] fileExistsAtPath:custom_icon_path]) {
+        NSImage *icon_image = [[[NSImage alloc] initWithContentsOfFile:custom_icon_path] autorelease];
+        [NSApplication sharedApplication].applicationIconImage = icon_image;
+        Py_RETURN_NONE;
+    }
+    return NULL;
+
+    } // autoreleasepool
+}
+
 static NSSound *beep_sound = nil;
 
 static void
-cleanup() {
+cleanup(void) {
     @autoreleasepool {
 
     if (dockMenu) [dockMenu release];
@@ -886,17 +985,6 @@ cleanup() {
     } // autoreleasepool
 }
 
-void
-cocoa_hide_window_title(void *w)
-{
-    @autoreleasepool {
-
-    NSWindow *window = (NSWindow*)w;
-    [window setTitleVisibility:NSWindowTitleHidden];
-
-    } // autoreleasepool
-}
-
 void
 cocoa_system_beep(const char *path) {
     if (!path) { NSBeep(); return; }
@@ -909,26 +997,32 @@ cocoa_system_beep(const char *path) {
     else NSBeep();
 }
 
-static PyMethodDef module_methods[] = {
-    {"cocoa_get_lang", (PyCFunction)cocoa_get_lang, METH_NOARGS, ""},
-    {"cocoa_set_global_shortcut", (PyCFunction)cocoa_set_global_shortcut, METH_VARARGS, ""},
-    {"cocoa_send_notification", (PyCFunction)cocoa_send_notification, METH_VARARGS, ""},
-    {"cocoa_set_notification_activated_callback", (PyCFunction)set_notification_activated_callback, METH_O, ""},
-    {"cocoa_set_url_handler", (PyCFunction)cocoa_set_url_handler, METH_VARARGS, ""},
-    {NULL, NULL, 0, NULL}        /* Sentinel */
-};
-
 static void
 uncaughtExceptionHandler(NSException *exception) {
     log_error("Unhandled exception in Cocoa: %s", [[exception description] UTF8String]);
     log_error("Stack trace:\n%s", [[exception.callStackSymbols description] UTF8String]);
 }
 
+void
+cocoa_set_uncaught_exception_handler(void) {
+    NSSetUncaughtExceptionHandler(&uncaughtExceptionHandler);
+}
+
+static PyMethodDef module_methods[] = {
+    {"cocoa_get_lang", (PyCFunction)cocoa_get_lang, METH_NOARGS, ""},
+    {"cocoa_set_global_shortcut", (PyCFunction)cocoa_set_global_shortcut, METH_VARARGS, ""},
+    {"cocoa_send_notification", (PyCFunction)cocoa_send_notification, METH_VARARGS, ""},
+    {"cocoa_set_notification_activated_callback", (PyCFunction)set_notification_activated_callback, METH_O, ""},
+    {"cocoa_set_url_handler", (PyCFunction)cocoa_set_url_handler, METH_VARARGS, ""},
+    {"cocoa_set_app_icon", (PyCFunction)cocoa_set_app_icon, METH_VARARGS, ""},
+    {"cocoa_set_dock_icon", (PyCFunction)cocoa_set_dock_icon, METH_VARARGS, ""},
+    {NULL, NULL, 0, NULL}        /* Sentinel */
+};
+
 bool
 init_cocoa(PyObject *module) {
-    memset(&global_shortcuts, 0, sizeof(global_shortcuts));
+    cocoa_clear_global_shortcuts();
     if (PyModule_AddFunctions(module, module_methods) != 0) return false;
     register_at_exit_cleanup_func(COCOA_CLEANUP_FUNC, cleanup);
-    NSSetUncaughtExceptionHandler(&uncaughtExceptionHandler);
     return true;
 }
