@@ -10,7 +10,6 @@
 #include "state.h"
 #include "emoji.h"
 #include "unicode-data.h"
-#include "charsets.h"
 #include "glyph-cache.h"
 
 #define MISSING_GLYPH (NUM_UNDERLINE_STYLES + 2)
@@ -71,6 +70,12 @@ typedef struct Canvas {
     unsigned current_cells, alloced_cells;
 } Canvas;
 
+typedef struct fallback_font_map {
+    const char *cell_text;
+    size_t font_idx;
+    UT_hash_handle hh;
+} fallback_font_map_t;
+
 typedef struct {
     FONTS_DATA_HEAD
     id_type id;
@@ -80,6 +85,7 @@ typedef struct {
     Font *fonts;
     Canvas canvas;
     GPUSpriteTracker sprite_tracker;
+    fallback_font_map_t *fallback_font_map;
 } FontGroup;
 
 static FontGroup* font_groups = NULL;
@@ -152,6 +158,15 @@ static void
 del_font_group(FontGroup *fg) {
     free(fg->canvas.buf); fg->canvas.buf = NULL; fg->canvas = (Canvas){0};
     fg->sprite_map = free_sprite_map(fg->sprite_map);
+    if (fg->fallback_font_map) {
+        fallback_font_map_t *current, *tmp;
+        HASH_ITER(hh, fg->fallback_font_map, current, tmp) {
+            free((void*)current->cell_text);
+            HASH_DEL(fg->fallback_font_map, current);
+            free(current);
+        }
+        fg->fallback_font_map = NULL;
+    }
     for (size_t i = 0; i < fg->fonts_count; i++) del_font(fg->fonts + i);
     free(fg->fonts); fg->fonts = NULL;
 }
@@ -331,6 +346,27 @@ python_send_to_gpu(FONTS_DATA_HANDLE fg, unsigned int x, unsigned int y, unsigne
     }
 }
 
+static void
+adjust_metric(unsigned int *metric, float adj, AdjustmentUnit unit, double dpi) {
+    if (adj == 0.f) return;
+    int a = 0;
+    switch (unit) {
+        case POINT:
+            a = ((long)round((adj * (dpi / 72.0)))); break;
+        case PERCENT:
+            *metric = (int)roundf((fabsf(adj) * (float)*metric) / 100.f); return;
+        case PIXEL:
+            a = (int)roundf(adj); break;
+    }
+    *metric = (a < 0 && -a > (int)*metric) ? 0 : *metric + a;
+}
+
+static unsigned int
+adjust_ypos(unsigned int pos, unsigned int cell_height, int adjustment) {
+    if (adjustment >= 0) adjustment = MIN(adjustment, (int)pos - 1);
+    else adjustment = MAX(adjustment, (int)pos - (int)cell_height + 1);
+    return pos - adjustment;
+}
 
 static void
 calc_cell_metrics(FontGroup *fg) {
@@ -338,18 +374,16 @@ calc_cell_metrics(FontGroup *fg) {
     cell_metrics(fg->fonts[fg->medium_font_idx].face, &cell_width, &cell_height, &baseline, &underline_position, &underline_thickness, &strikethrough_position, &strikethrough_thickness);
     if (!cell_width) fatal("Failed to calculate cell width for the specified font");
     unsigned int before_cell_height = cell_height;
-    int cw = cell_width, ch = cell_height;
-    if (OPT(adjust_line_height_px) != 0) ch += OPT(adjust_line_height_px);
-    if (OPT(adjust_line_height_frac) != 0.f) ch = (int)(ch * OPT(adjust_line_height_frac));
-    if (OPT(adjust_column_width_px != 0)) cw += OPT(adjust_column_width_px);
-    if (OPT(adjust_column_width_frac) != 0.f) cw = (int)(cw * OPT(adjust_column_width_frac));
+    unsigned int cw = cell_width, ch = cell_height;
+    adjust_metric(&cw, OPT(cell_width).val, OPT(cell_width).unit, fg->logical_dpi_x);
+    adjust_metric(&ch, OPT(cell_height).val, OPT(cell_height).unit, fg->logical_dpi_y);
 #define MAX_DIM 1000
 #define MIN_WIDTH 2
 #define MIN_HEIGHT 4
     if (cw >= MIN_WIDTH && cw <= MAX_DIM) cell_width = cw;
-    else log_error("Cell width invalid after adjustment, ignoring adjust_column_width");
+    else log_error("Cell width invalid after adjustment, ignoring modify_font cell_width");
     if (ch >= MIN_HEIGHT && ch <= MAX_DIM) cell_height = ch;
-    else log_error("Cell height invalid after adjustment, ignoring adjust_line_height");
+    else log_error("Cell height invalid after adjustment, ignoring modify_font cell_height");
     int line_height_adjustment = cell_height - before_cell_height;
     if (cell_height < MIN_HEIGHT) fatal("Line height too small: %u", cell_height);
     if (cell_height > MAX_DIM) fatal("Line height too large: %u", cell_height);
@@ -358,9 +392,24 @@ calc_cell_metrics(FontGroup *fg) {
 #undef MIN_WIDTH
 #undef MIN_HEIGHT
 #undef MAX_DIM
+
+    unsigned int baseline_before = baseline;
+#define A(which, dpi) adjust_metric(&which, OPT(which).val, OPT(which).unit, fg->logical_dpi_##dpi);
+    A(underline_thickness, y); A(underline_position, y); A(strikethrough_thickness, y); A(strikethrough_position, y); A(baseline, y);
+#undef A
+
+    if (baseline_before != baseline) {
+        int adjustment = baseline - baseline_before;
+        baseline = adjust_ypos(baseline_before, cell_height, adjustment);
+        underline_position = adjust_ypos(underline_position, cell_height, adjustment);
+        strikethrough_position = adjust_ypos(strikethrough_position, cell_height, adjustment);
+    }
+
     underline_position = MIN(cell_height - 1, underline_position);
     // ensure there is at least a couple of pixels available to render styled underlines
-    while (underline_position > baseline + 1 && cell_height - underline_position < 2) underline_position--;
+    // there should be at least one pixel on either side of the underline_position
+    if (underline_position > baseline + 1 && underline_position > cell_height - 1)
+      underline_position = MAX(baseline + 1, cell_height - 1);
     if (line_height_adjustment > 1) {
         baseline += MIN(cell_height - 1, (unsigned)line_height_adjustment / 2);
         underline_position += MIN(cell_height - 1, (unsigned)line_height_adjustment / 2);
@@ -417,12 +466,22 @@ output_cell_fallback_data(CPUCell *cell, bool bold, bool italic, bool emoji_pres
     printf("\n");
 }
 
+PyObject*
+iter_fallback_faces(FONTS_DATA_HANDLE fgh, ssize_t *idx) {
+    FontGroup *fg = (FontGroup*)fgh;
+    if (*idx + 1 < (ssize_t)fg->fallback_fonts_count) {
+        *idx += 1;
+        return fg->fonts[fg->first_fallback_font_idx + *idx].face;
+    }
+    return NULL;
+}
+
 static ssize_t
 load_fallback_font(FontGroup *fg, CPUCell *cell, bool bold, bool italic, bool emoji_presentation) {
     if (fg->fallback_fonts_count > 100) { log_error("Too many fallback fonts"); return MISSING_FONT; }
     ssize_t f;
 
-    if (bold) f = fg->italic_font_idx > 0 ? fg->bi_font_idx : fg->bold_font_idx;
+    if (bold) f = italic ? fg->bi_font_idx : fg->bold_font_idx;
     else f = italic ? fg->italic_font_idx : fg->medium_font_idx;
     if (f < 0) f = fg->medium_font_idx;
 
@@ -430,6 +489,7 @@ load_fallback_font(FontGroup *fg, CPUCell *cell, bool bold, bool italic, bool em
     if (face == NULL) { PyErr_Print(); return MISSING_FONT; }
     if (face == Py_None) { Py_DECREF(face); return MISSING_FONT; }
     if (global_state.debug_font_fallback) output_cell_fallback_data(cell, bold, italic, emoji_presentation, face, true);
+    if (PyLong_Check(face)) { ssize_t ans = fg->first_fallback_font_idx + PyLong_AsSsize_t(face); Py_DECREF(face); return ans; }
     set_size_for_face(face, fg->cell_height, true, (FONTS_DATA_HANDLE)fg);
 
     ensure_space_for(fg, fonts, Font, fg->fonts_count + 1, fonts_capacity, 5, true);
@@ -461,17 +521,26 @@ fallback_font(FontGroup *fg, CPUCell *cpu_cell, GPUCell *gpu_cell) {
     bool bold = gpu_cell->attrs.bold;
     bool italic = gpu_cell->attrs.italic;
     bool emoji_presentation = has_emoji_presentation(cpu_cell, gpu_cell);
-
-    // Check if one of the existing fallback fonts has this text
-    for (size_t i = 0, j = fg->first_fallback_font_idx; i < fg->fallback_fonts_count; i++, j++)  {
-        Font *ff = fg->fonts +j;
-        if (ff->bold == bold && ff->italic == italic && ff->emoji_presentation == emoji_presentation && has_cell_text(ff, cpu_cell)) {
-            if (global_state.debug_font_fallback) output_cell_fallback_data(cpu_cell, bold, italic, emoji_presentation, ff->face, false);
-            return j;
+    char style = emoji_presentation ? 'a' : 'A';
+    if (bold) style += italic ? 3 : 2; else style += italic ? 1 : 0;
+    char cell_text[8 + arraysz(cpu_cell->cc_idx) * 4] = {style};
+    const size_t cell_text_len = 1 + cell_as_utf8(cpu_cell, true, cell_text + 1, ' ');
+    if (fg->fallback_font_map) {
+        fallback_font_map_t *s;
+        HASH_FIND_STR(fg->fallback_font_map, cell_text, s);
+        /* printf("cache %s\n", (s ? "hit" : "miss")); */
+        if (s) return s->font_idx;
+    }
+    ssize_t idx = load_fallback_font(fg, cpu_cell, bold, italic, emoji_presentation);
+    fallback_font_map_t *ffm = calloc(1, sizeof(fallback_font_map_t));
+    if (ffm) {
+        ffm->font_idx = idx;
+        ffm->cell_text = strndup(cell_text, cell_text_len);
+        if (ffm->cell_text) {
+            HASH_ADD_KEYPTR(hh, fg->fallback_font_map, ffm->cell_text, cell_text_len, ffm);
         }
     }
-
-    return load_fallback_font(fg, cpu_cell, bold, italic, emoji_presentation);
+    return idx;
 }
 
 static ssize_t
@@ -500,14 +569,15 @@ START_ALLOW_CASE_RANGE
     switch(cpu_cell->ch) {
         case 0:
         case ' ':
+        case 0x2002:  // en-space
         case '\t':
+        case IMAGE_PLACEHOLDER_CHAR:
             return BLANK_FONT;
         case 0x2500 ... 0x2573:
         case 0x2574 ... 0x259f:
         case 0x2800 ... 0x28ff:
-        case 0xe0b0 ... 0xe0bf:  // powerline box drawing
-        case 0x1fb00 ... 0x1fb8b:  // symbols for legacy computing
-        case 0x1fba0 ... 0x1fbae:
+        case 0xe0b0 ... 0xe0bf:    // powerline box drawing
+        case 0x1fb00 ... 0x1fbae:  // symbols for legacy computing
             return BOX_FONT;
         default:
             *is_emoji_presentation = has_emoji_presentation(cpu_cell, gpu_cell);
@@ -543,13 +613,11 @@ START_ALLOW_CASE_RANGE
         case 0x2500 ... 0x259f:
             return ch - 0x2500; // IDs from 0x00 to 0x9f
         case 0xe0b0 ... 0xe0d4:
-            return 0xa0 + ch - 0xe0b0; // IDs from 0xa0 to 0xc4
-        case 0x1fb00 ... 0x1fb8b:
-            return 0xc5 + ch - 0x1fb00; // IDs from 0xc5 to 0x150
-        case 0x1fba0 ... 0x1fbae:  // IDs from 0x151 to 0x15f
-            return 0x151 + ch - 0x1fba0;
+            return 0xa0 + ch - 0xe0b0;  // IDs from 0xa0 to 0xc4
         case 0x2800 ... 0x28ff:
-            return 0x160 + ch - 0x2800;
+            return 0xc5 + ch - 0x2800; // IDs from 0xc5 to 0x1c4
+        case 0x1fb00 ... 0x1fbae:
+            return 0x1c5 + ch - 0x1fb00;  // IDs from 0x1c5 to 0x273
         default:
             return 0xffff;
     }
@@ -928,8 +996,14 @@ group_iosevka(Font *font, hb_font_t *hbf) {
         const LigatureType current = ligature_types[G(glyph_idx)];
         const LigatureType after = is_last_glyph ? LIGATURE_UNKNOWN : ligature_types[G(glyph_idx + 1)];
         bool end_current_group = false;
-        if (current_group->num_glyphs && is_iosevka_lig_ender(before, current, after)) {
-            end_current_group = true;
+        if (current_group->num_glyphs) {
+            if (is_iosevka_lig_ender(before, current, after)) end_current_group = true;
+            else {
+                if (!current_group->num_cells && !current_group->has_special_glyph) {
+                    if (is_iosevka_lig_starter(before, current, after)) current_group->has_special_glyph = true;
+                    else end_current_group = true;
+                }
+            }
         }
         if (!current_group->num_glyphs++) {
             if (is_iosevka_lig_starter(before, current, after)) current_group->has_special_glyph = true;
@@ -954,7 +1028,7 @@ group_iosevka(Font *font, hb_font_t *hbf) {
             }
             current_group->num_cells += num_cells_consumed;
         }
-        if (end_current_group) G(group_idx)++;
+        if (end_current_group && current_group->num_cells) G(group_idx)++;
         G(glyph_idx)++;
     }
 }
@@ -1017,9 +1091,9 @@ group_normal(Font *font, hb_font_t *hbf) {
 #if 0
         char ch[8] = {0};
         encode_utf8(G(current_cell_data).current_codepoint, ch);
-        printf("\x1b[32m→ %s\x1b[m glyph_idx: %zu glyph_id: %u (%s) group_idx: %zu cluster: %u -> %u is_special: %d\n"
+        printf("\x1b[32m→ %s\x1b[m glyph_idx: %zu glyph_id: %u group_idx: %zu cluster: %u -> %u is_special: %d\n"
                 "  num_codepoints_used_by_glyph: %u current_group: (%u cells, %u glyphs) add_to_current_group: %d\n",
-                ch, G(glyph_idx), glyph_id, glyph_name, G(group_idx), cluster, next_cluster, is_special,
+                ch, G(glyph_idx), glyph_id, G(group_idx), cluster, next_cluster, is_special,
                 num_codepoints_used_by_glyph, current_group->num_cells, current_group->num_glyphs, add_to_current_group);
 #endif
 
@@ -1280,7 +1354,7 @@ render_line(FONTS_DATA_HANDLE fg_, Line *line, index_type lnum, Cursor *cursor,
             unsigned int num_spaces = 0;
             while (
                     i + num_spaces + 1 < line->xnum
-                    && line->cpu_cells[i+num_spaces+1].ch == ' '
+                    && (line->cpu_cells[i+num_spaces+1].ch == ' ' || line->cpu_cells[i+num_spaces+1].ch == 0x2002)  // space or en-space
                     && num_spaces < MAX_NUM_EXTRA_GLYPHS_PUA
                     && num_spaces + 1 < desired_cells
                     ) {
@@ -1482,7 +1556,7 @@ sprite_map_set_layout(PyObject UNUSED *self, PyObject *args) {
 static PyObject*
 test_sprite_position_for(PyObject UNUSED *self, PyObject *args) {
     int error;
-    FREE_AFTER_FUNCTION glyph_index *glyphs = calloc(PyTuple_GET_SIZE(args), sizeof(glyph_index));
+    RAII_ALLOC(glyph_index, glyphs, calloc(PyTuple_GET_SIZE(args), sizeof(glyph_index)));
     for (Py_ssize_t i = 0; i < PyTuple_GET_SIZE(args); i++) {
         if (!PyLong_Check(PyTuple_GET_ITEM(args, i))) {
             PyErr_SetString(PyExc_TypeError, "glyph indices must be integers");
