@@ -1,5 +1,4 @@
-#!/usr/bin/env python3
-# vim:fileencoding=utf-8
+#!/usr/bin/env python
 # License: GPL v3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
 import re
@@ -7,15 +6,23 @@ from functools import lru_cache
 from typing import Dict, Generator, List, Optional, Tuple, cast
 
 from kitty.fast_data_types import (
-    FC_DUAL, FC_MONO, FC_SLANT_ITALIC, FC_SLANT_ROMAN, FC_WEIGHT_BOLD,
-    FC_WEIGHT_REGULAR, FC_WIDTH_NORMAL, fc_list, fc_match as fc_match_impl,
-    fc_match_postscript_name, parse_font_feature
+    FC_DUAL,
+    FC_MONO,
+    FC_SLANT_ITALIC,
+    FC_SLANT_ROMAN,
+    FC_WEIGHT_BOLD,
+    FC_WEIGHT_REGULAR,
+    FC_WIDTH_NORMAL,
+    fc_list,
+    fc_match_postscript_name,
+    parse_font_feature,
 )
-from kitty.options_stub import Options
+from kitty.fast_data_types import fc_match as fc_match_impl
+from kitty.options.types import Options
 from kitty.typing import FontConfigPattern
 from kitty.utils import log_error
 
-from . import ListedFont, FontFeature
+from . import FontFeature, ListedFont
 
 attr_map = {(False, False): 'font_family',
             (True, False): 'bold_font',
@@ -29,7 +36,7 @@ FontMap = Dict[str, Dict[str, List[FontConfigPattern]]]
 def create_font_map(all_fonts: Tuple[FontConfigPattern, ...]) -> FontMap:
     ans: FontMap = {'family_map': {}, 'ps_map': {}, 'full_map': {}}
     for x in all_fonts:
-        if 'path' not in x:
+        if not x.get('path'):
             continue
         f = (x.get('family') or '').lower()
         full = (x.get('full_name') or '').lower()
@@ -45,7 +52,9 @@ def all_fonts_map(monospaced: bool = True) -> FontMap:
     if monospaced:
         ans = fc_list(FC_DUAL) + fc_list(FC_MONO)
     else:
-        ans = fc_list()
+        # allow non-monospaced and bitmapped fonts as these are used for
+        # symbol_map
+        ans = fc_list(-1, True)
     return create_font_map(ans)
 
 
@@ -57,7 +66,7 @@ def list_fonts() -> Generator[ListedFont, None, None]:
             if fn_:
                 fn = str(fn_)
             else:
-                fn = (f + ' ' + str(fd.get('style', ''))).strip()
+                fn = f'{f} {fd.get("style", "")}'.strip()
             is_mono = fd.get('spacing') in ('MONO', 'DUAL')
             yield {'family': f, 'full_name': fn, 'postscript_name': str(fd.get('postscript_name', '')), 'is_monospace': is_mono}
 
@@ -71,7 +80,7 @@ def fc_match(family: str, bold: bool, italic: bool, spacing: int = FC_MONO) -> F
     return fc_match_impl(family, bold, italic, spacing)
 
 
-def find_font_features(postscript_name: str) -> Tuple[str, ...]:
+def find_font_features(postscript_name: str) -> Tuple[FontFeature, ...]:
     pat = fc_match_postscript_name(postscript_name)
 
     if pat.get('postscript_name') != postscript_name or 'fontfeatures' not in pat:
@@ -82,7 +91,7 @@ def find_font_features(postscript_name: str) -> Tuple[str, ...]:
         try:
             parsed = parse_font_feature(feat)
         except ValueError:
-            log_error('Ignoring invalid font feature: {}'.format(feat))
+            log_error(f'Ignoring invalid font feature: {feat}')
         else:
             features.append(FontFeature(feat, parsed))
 
@@ -114,8 +123,13 @@ def find_best_match(family: str, bold: bool = False, italic: bool = False, monos
         return candidates[0]
 
     # Use fc-match to see if we can find a monospaced font that matches family
-    for spacing in (FC_MONO, FC_DUAL):
-        possibility = fc_match(family, False, False, spacing)
+    # When aliases are defined, spacing can cause the incorrect font to be
+    # returned, so check with and without spacing and use the one that matches.
+    mono_possibility = fc_match(family, False, False, FC_MONO)
+    dual_possibility = fc_match(family, False, False, FC_DUAL)
+    any_possibility = fc_match(family, False, False, 0)
+    tries = (dual_possibility, mono_possibility) if any_possibility == dual_possibility else (mono_possibility, dual_possibility)
+    for possibility in tries:
         for key, map_key in (('postscript_name', 'ps_map'), ('full_name', 'full_map'), ('family', 'family_map')):
             val: Optional[str] = cast(Optional[str], possibility.get(key))
             if val:
