@@ -210,6 +210,25 @@ init_ft_face(Face *self, PyObject *path, int hinting, int hintstyle, FONTS_DATA_
     return true;
 }
 
+static void*
+set_load_error(const char *path, int error) {
+    char buf[2048];
+    snprintf(buf, sizeof(buf), "Failed to load face from path: %s with error:", path);
+    set_freetype_error(buf, error);
+    return NULL;
+}
+
+bool
+face_equals_descriptor(PyObject *face_, PyObject *descriptor) {
+    Face *face = (Face*)face_;
+    PyObject *t = PyDict_GetItemString(descriptor, "path");
+    if (!t) return false;
+    if (PyObject_RichCompareBool(face->path, t, Py_EQ) != 1) return false;
+    t = PyDict_GetItemString(descriptor, "index");
+    if (t && PyLong_AsLong(t) != face->index) return false;
+    return true;
+}
+
 PyObject*
 face_from_descriptor(PyObject *descriptor, FONTS_DATA_HANDLE fg) {
 #define D(key, conv, missing_ok) { \
@@ -230,7 +249,7 @@ face_from_descriptor(PyObject *descriptor, FONTS_DATA_HANDLE fg) {
     Face *self = (Face *)Face_Type.tp_alloc(&Face_Type, 0);
     if (self != NULL) {
         int error = FT_New_Face(library, path, index, &(self->face));
-        if(error) { set_freetype_error("Failed to load face, with error:", error); Py_CLEAR(self); return NULL; }
+        if(error) { Py_CLEAR(self); return set_load_error(path, error); }
         if (!init_ft_face(self, PyDict_GetItemString(descriptor, "path"), hinting, hint_style, fg)) { Py_CLEAR(self); return NULL; }
     }
     return (PyObject*)self;
@@ -241,7 +260,7 @@ native_face_from_path(const char *path, int index) {
     int error;
     FT_Face ans;
     error = FT_New_Face(library, path, index, &ans);
-    if (error) { set_freetype_error("Failed to load face, with error:", error); return NULL; }
+    if (error) return set_load_error(path, error);
     return ans;
 }
 
@@ -251,7 +270,7 @@ face_from_path(const char *path, int index, FONTS_DATA_HANDLE fg) {
     if (ans == NULL) return NULL;
     int error;
     error = FT_New_Face(library, path, index, &ans->face);
-    if (error) { set_freetype_error("Failed to load face, with error:", error); ans->face = NULL; return NULL; }
+    if (error) { ans->face = NULL; return set_load_error(path, error); }
     if (!init_ft_face(ans, Py_None, true, 3, fg)) { Py_CLEAR(ans); return NULL; }
     return (PyObject*)ans;
 }
@@ -277,7 +296,6 @@ repr(Face *self) {
     );
 }
 
-
 const char*
 postscript_name_for_face(const PyObject *face_) {
     const Face *self = (const Face*)face_;
@@ -298,21 +316,11 @@ calc_cell_width(Face *self) {
 }
 
 
-static unsigned int
-adjust_ypos(unsigned int pos, unsigned int cell_height, int adjustment) {
-    if (adjustment >= 0) adjustment = MIN(adjustment, (int)pos - 1);
-    else adjustment = MAX(adjustment, (int)pos - (int)cell_height + 1);
-    return pos - adjustment;
-}
-
 void
 cell_metrics(PyObject *s, unsigned int* cell_width, unsigned int* cell_height, unsigned int* baseline, unsigned int* underline_position, unsigned int* underline_thickness, unsigned int* strikethrough_position, unsigned int* strikethrough_thickness) {
     Face *self = (Face*)s;
     *cell_width = calc_cell_width(self);
     *cell_height = calc_cell_height(self, true);
-    int baseline_offset = 0;
-    if (OPT(adjust_baseline_px) != 0) baseline_offset = OPT(adjust_baseline_px);
-    else if (OPT(adjust_baseline_frac) != 0) baseline_offset = (int)(*cell_height * OPT(adjust_baseline_frac));
     *baseline = font_units_to_pixels_y(self, self->ascender);
     *underline_position = MIN(*cell_height - 1, (unsigned int)font_units_to_pixels_y(self, MAX(0, self->ascender - self->underline_position)));
     *underline_thickness = MAX(1, font_units_to_pixels_y(self, self->underline_thickness));
@@ -327,11 +335,6 @@ cell_metrics(PyObject *s, unsigned int* cell_width, unsigned int* cell_height, u
     } else {
       *strikethrough_thickness = *underline_thickness;
     }
-    if (baseline_offset) {
-        *baseline = adjust_ypos(*baseline, *cell_height, baseline_offset);
-        *underline_position = adjust_ypos(*underline_position, *cell_height, baseline_offset);
-        *strikethrough_position = adjust_ypos(*strikethrough_position, *cell_height, baseline_offset);
-    }
 }
 
 unsigned int
@@ -730,7 +733,6 @@ static PyMethodDef methods[] = {
     {NULL}  /* Sentinel */
 };
 
-
 PyTypeObject Face_Type = {
     PyVarObject_HEAD_INIT(NULL, 0)
     .tp_name = "fast_data_types.Face",
