@@ -58,7 +58,7 @@
 // National Replacement Character Set Mode
 #define DECNRCM (42 << 5)
 
-// Xterm mouse protocol
+// xterm mouse protocol
 #define MOUSE_BUTTON_TRACKING (1000 << 5)
 #define MOUSE_MOTION_TRACKING  (1002 << 5)
 #define MOUSE_MOVE_TRACKING (1003 << 5)
@@ -84,3 +84,6 @@
 
 // Pending updates mode
 #define PENDING_UPDATE (2026 << 5)
+
+// Handle Ctrl-C/Ctrl-Z mode
+#define HANDLE_TERMIOS_SIGNALS (19997 << 5)
