@@ -1207,6 +1207,14 @@ class KeyDefinition(BaseDefinition):
     def is_suitable_for_global_shortcut(self) -> bool:
         return not self.options.when_focus_on and not self.options.mode and not self.options.new_mode and not self.is_sequence
 
+    @property
+    def full_key_sequence_to_trigger(self) -> Tuple[SingleKey, ...]:
+        return (self.trigger,) + self.rest
+
+    @property
+    def unique_identity_within_keymap(self) -> Tuple[Tuple[SingleKey, ...], str]:
+        return self.full_key_sequence_to_trigger, self.options.when_focus_on
+
     def __repr__(self) -> str:
         return self.pretty_repr('is_sequence', 'trigger', 'rest', 'options')
 
@@ -1234,7 +1242,7 @@ class KeyboardMode:
 
     on_unknown: OnUnknown = get_args(OnUnknown)[0]
     on_action : OnAction = get_args(OnAction)[0]
-    is_sequence: bool = False
+    sequence_keys: Optional[List[defines.KeyEvent]] = None
 
     def __init__(self, name: str = '') -> None:
         self.name = name
