@@ -1,16 +1,11 @@
-#!/usr/bin/env python3
-# vim:fileencoding=utf-8
+#!/usr/bin/env python
 # License: GPL v3 Copyright: 2017, Kovid Goyal <kovid at kovidgoyal.net>
 
 import re
 from contextlib import suppress
-from typing import Optional, NamedTuple
+from typing import Optional
 
-
-class Color(NamedTuple):
-    red: int
-    green: int
-    blue: int
+from .fast_data_types import Color
 
 
 def alpha_blend_channel(top_color: int, bottom_color: int, alpha: float) -> int:
@@ -34,7 +29,7 @@ def parse_single_color(c: str) -> int:
 def parse_sharp(spec: str) -> Optional[Color]:
     if len(spec) in (3, 6, 9, 12):
         part_len = len(spec) // 3
-        colors = re.findall(r'[a-fA-F0-9]{%d}' % part_len, spec)
+        colors = re.findall(fr'[a-fA-F0-9]{{{part_len}}}', spec)
         return Color(*map(parse_single_color, colors))
     return None
 
@@ -51,15 +46,15 @@ def color_from_int(x: int) -> Color:
 
 
 def color_as_int(x: Color) -> int:
-    return x.red << 16 | x.green << 8 | x.blue
+    return int(x)
 
 
 def color_as_sharp(x: Color) -> str:
-    return '#{:02x}{:02x}{:02x}'.format(*x)
+    return x.as_sharp
 
 
 def color_as_sgr(x: Color) -> str:
-    return ':2:{}:{}:{}'.format(*x)
+    return x.as_sgr
 
 
 def to_color(raw: str, validate: bool = False) -> Optional[Color]:
@@ -72,10 +67,10 @@ def to_color(raw: str, validate: bool = False) -> Optional[Color]:
     with suppress(Exception):
         if raw.startswith('#'):
             val = parse_sharp(raw[1:])
-        elif raw[:4].lower() == 'rgb:':
+        elif raw.startswith('rgb:'):
             val = parse_rgb(raw[4:])
     if val is None and validate:
-        raise ValueError('Invalid color name: {}'.format(raw))
+        raise ValueError(f'Invalid color name: {raw}')
     return val
 
 
@@ -838,8 +833,8 @@ color_names = {
 
 if __name__ == '__main__':
     # Read RGB color table from specified rgb.txt file
-    import sys
     import pprint
+    import sys
     data = {}
     with open(sys.argv[-1]) as f:
         for line in f:
