@@ -14,7 +14,7 @@ typedef enum ScrollTypes { SCROLL_LINE = -999999, SCROLL_PAGE, SCROLL_FULL } Scr
 
 typedef struct {
     bool mLNM, mIRM, mDECTCEM, mDECSCNM, mDECOM, mDECAWM, mDECCOLM, mDECARM, mDECCKM,
-         mBRACKETED_PASTE, mFOCUS_TRACKING, mDECSACE;
+         mBRACKETED_PASTE, mFOCUS_TRACKING, mDECSACE, mHANDLE_TERMIOS_SIGNALS;
     MouseTrackingMode mouse_tracking_mode;
     MouseTrackingProtocol mouse_tracking_protocol;
     bool eight_bit_controls;  // S8C1T
@@ -25,7 +25,7 @@ typedef struct {
     bool in_left_half_of_cell;
 } SelectionBoundary;
 
-typedef enum SelectionExtendModes { EXTEND_CELL, EXTEND_WORD, EXTEND_LINE, EXTEND_LINE_FROM_POINT } SelectionExtendMode;
+typedef enum SelectionExtendModes { EXTEND_CELL, EXTEND_WORD, EXTEND_LINE, EXTEND_LINE_FROM_POINT, EXTEND_WORD_AND_LINE_FROM_POINT } SelectionExtendMode;
 
 typedef struct {
     index_type x, x_limit;
@@ -39,7 +39,7 @@ typedef struct {
 typedef struct {
     SelectionBoundary start, end, input_start, input_current;
     unsigned int start_scrolled_by, end_scrolled_by;
-    bool rectangle_select, adjusting_start;
+    bool rectangle_select, adjusting_start, is_hyperlink;
     IterationData last_rendered;
     int sort_y, sort_x;
     struct {
@@ -68,17 +68,27 @@ typedef struct {
 
 
 typedef struct {
+    PyObject *overlay_text;
     CPUCell *cpu_cells;
     GPUCell *gpu_cells;
+    index_type xstart, ynum, xnum, cursor_x, text_len;
     bool is_active;
-    index_type xstart, ynum, xnum;
+    bool is_dirty;
+    struct {
+        CPUCell *cpu_cells;
+        GPUCell *gpu_cells;
+        Cursor cursor;
+    } original_line;
+    struct {
+        index_type x, y;
+    } last_ime_pos;
 } OverlayLine;
 
 typedef struct {
     PyObject_HEAD
 
     unsigned int columns, lines, margin_top, margin_bottom, charset, scrolled_by;
-    double pending_scroll_pixels;
+    double pending_scroll_pixels_x, pending_scroll_pixels_y;
     CellPixelSize cell_size;
     OverlayLine overlay_line;
     id_type window_id;
@@ -149,6 +159,14 @@ typedef struct {
         index_type y;
         bool is_set;
     } last_visited_prompt;
+    PyObject *last_reported_cwd;
+    struct {
+        hyperlink_id_type id;
+        index_type x, y;
+    } current_hyperlink_under_mouse;
+    struct {
+        uint8_t stack[16], count;
+    } main_pointer_shape_stack, alternate_pointer_shape_stack;
 } Screen;
 
 
@@ -196,6 +214,7 @@ void screen_cursor_forward(Screen *self, unsigned int count/*=1*/);
 void screen_cursor_down1(Screen *self, unsigned int count/*=1*/);
 void screen_cursor_up1(Screen *self, unsigned int count/*=1*/);
 void screen_cursor_to_line(Screen *screen, unsigned int line);
+MouseShape screen_pointer_shape(Screen *self);
 void screen_insert_lines(Screen *self, unsigned int count/*=1*/);
 void screen_delete_lines(Screen *self, unsigned int count/*=1*/);
 void screen_repeat_character(Screen *self, unsigned int count);
@@ -207,7 +226,7 @@ void screen_handle_cmd(Screen *, PyObject *cmd);
 void screen_push_colors(Screen *, unsigned int);
 void screen_pop_colors(Screen *, unsigned int);
 void screen_report_color_stack(Screen *);
-void screen_handle_print(Screen *, PyObject *cmd);
+void screen_handle_kitty_dcs(Screen *, const char *callback_name, PyObject *cmd);
 void screen_designate_charset(Screen *, uint32_t which, uint32_t as);
 void screen_use_latin1(Screen *, bool);
 void set_title(Screen *self, PyObject*);
@@ -240,6 +259,7 @@ typedef struct SelectionUpdate {
 } SelectionUpdate;
 void screen_update_selection(Screen *self, index_type x, index_type y, bool in_left_half, SelectionUpdate upd);
 bool screen_history_scroll(Screen *self, int amt, bool upwards);
+PyObject* as_text_history_buf(HistoryBuf *self, PyObject *args, ANSIBuf *output);
 Line* screen_visual_line(Screen *self, index_type y);
 unsigned long screen_current_char_width(Screen *self);
 void screen_mark_url(Screen *self, index_type start_x, index_type start_y, index_type end_x, index_type end_y);
@@ -253,15 +273,18 @@ void screen_dirty_sprite_positions(Screen *self);
 void screen_rescale_images(Screen *self);
 void screen_report_size(Screen *, unsigned int which);
 void screen_manipulate_title_stack(Screen *, unsigned int op, unsigned int which);
-void screen_draw_overlay_text(Screen *self, const char *utf8_text);
+bool screen_is_overlay_active(Screen *self);
+void screen_update_overlay_text(Screen *self, const char *utf8_text);
 void screen_set_key_encoding_flags(Screen *self, uint32_t val, uint32_t how);
 void screen_push_key_encoding_flags(Screen *self, uint32_t val);
 void screen_pop_key_encoding_flags(Screen *self, uint32_t num);
 uint8_t screen_current_key_encoding_flags(Screen *self);
 void screen_report_key_encoding_flags(Screen *self);
-bool screen_detect_url(Screen *screen, unsigned int x, unsigned int y);
+int screen_detect_url(Screen *screen, unsigned int x, unsigned int y);
 int screen_cursor_at_a_shell_prompt(const Screen *);
 bool screen_fake_move_cursor_to_position(Screen *, index_type x, index_type y);
+bool screen_send_signal_for_key(Screen *, char key);
+bool get_line_edge_colors(Screen *self, color_type *left, color_type *right);
 #define DECLARE_CH_SCREEN_HANDLER(name) void screen_##name(Screen *screen);
 DECLARE_CH_SCREEN_HANDLER(bell)
 DECLARE_CH_SCREEN_HANDLER(backspace)
