@@ -1,22 +1,21 @@
-#!/usr/bin/env python3
-# vim:fileencoding=utf-8
+#!/usr/bin/env python
 # License: GPLv3 Copyright: 2019, Kovid Goyal <kovid at kovidgoyal.net>
 
 import os
 import subprocess
 import time
 from contextlib import suppress
-from typing import Dict, NamedTuple, Optional
+from typing import Dict, NamedTuple, Optional, Tuple
 from urllib.request import urlopen
 
 from .config import atomic_save
-from .constants import Version, cache_dir, kitty_exe, version
+from .constants import Version, cache_dir, clear_handled_signals, kitty_exe, version, website_url
 from .fast_data_types import add_timer, get_boss, monitor_pid
 from .notify import notify
 from .utils import log_error, open_url
 
-CHANGELOG_URL = 'https://sw.kovidgoyal.net/kitty/changelog.html'
-RELEASED_VERSION_URL = 'https://sw.kovidgoyal.net/kitty/current-version.txt'
+CHANGELOG_URL = website_url('changelog')
+RELEASED_VERSION_URL = website_url() + 'current-version.txt'
 CHECK_INTERVAL = 24 * 60 * 60.
 
 
@@ -74,7 +73,7 @@ def read_cache() -> Dict[Version, Notification]:
     return notified_versions
 
 
-def already_notified(version: tuple) -> bool:
+def already_notified(version: Tuple[int, int, int]) -> bool:
     notified_versions = read_cache()
     return version in notified_versions
 
@@ -90,7 +89,9 @@ def save_notification(version: Version) -> None:
     for version in sorted(notified_versions):
         n = notified_versions[version]
         lines.append('{},{},{}'.format(
-            '.'.join(map(str, n.version)), n.time_of_last_notification, n.count))
+            '.'.join(map(str, n.version)),
+            n.time_of_last_notification,
+            n.notification_count))
     atomic_save('\n'.join(lines).encode('utf-8'), version_notification_log())
 
 
@@ -102,27 +103,31 @@ def process_current_release(raw: str) -> None:
 
 
 def run_worker() -> None:
-    import time
     import random
+    import time
     time.sleep(random.randint(1000, 4000) / 1000)
     with suppress(BrokenPipeError):  # happens if parent process is killed before us
         print(get_released_version())
 
 
-def update_check(timer_id: Optional[int] = None) -> bool:
+def update_check() -> bool:
     try:
         p = subprocess.Popen([
             kitty_exe(), '+runpy',
             'from kitty.update_check import run_worker; run_worker()'
-        ], stdout=subprocess.PIPE)
+        ], stdout=subprocess.PIPE, preexec_fn=clear_handled_signals)
     except OSError as e:
-        log_error('Failed to run kitty for update check, with error: {}'.format(e))
+        log_error(f'Failed to run kitty for update check, with error: {e}')
         return False
     monitor_pid(p.pid)
     get_boss().set_update_check_process(p)
     return True
 
 
+def update_check_callback(timer_id: Optional[int]) -> None:
+    update_check()
+
+
 def run_update_check(interval: float = CHECK_INTERVAL) -> None:
     if update_check():
-        add_timer(update_check, interval)
+        add_timer(update_check_callback, interval)
